'use strict';

var _Object$getOwnPropertyNames = require('babel-runtime/core-js/object/get-own-property-names')['default'];

var _Object$defineProperty = require('babel-runtime/core-js/object/define-property')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = model;
var blacklist = {
  constructorProps: ['length', 'name', 'arguments', 'caller', 'prototype'],
  prototypeProps: ['constructor'],
  hookProps: ['beforeValidate', 'afterValidate', 'beforeBulkCreate', 'beforeBulkDestroy', 'beforeBulkUpdate', 'afterBulkCreate', 'afterBulkDestory', 'afterBulkUpdate', 'beforeCreate', 'beforeDestroy', 'beforeUpdate', 'afterCreate', 'afterDestroy', 'afterUpdate']
};

function model() {

  // Return decorated Model
  return function (Model) {

    // Ensure constructor properties are enumerable
    // to access props in parent class
    _Object$getOwnPropertyNames(Model).filter(function (prop) {
      return blacklist.constructorProps.indexOf(prop) === -1;
    }).forEach(function (prop) {
      return _Object$defineProperty(Model, prop, { enumerable: true });
    });

    // Ensure prototype properties are enumerable
    // to access props in parent class
    _Object$getOwnPropertyNames(Model.prototype).filter(function (prop) {
      return blacklist.prototypeProps.indexOf(prop) === -1;
    }).forEach(function (prop) {
      return _Object$defineProperty(Model.prototype, prop, { enumerable: true });
    });

    // Set "definition" getter
    // to create Seqelize Model Definition and return arguments for "Sequelize.define" function
    // See http://sequelize.readthedocs.org/en/latest/docs/models-definition/
    Object.defineProperty(Model.prototype, 'definition', {
      get: function get() {

        // Set defaults
        var modelName = this.constructor.name;
        var attributes = {};
        var options = { classMethods: {}, instanceMethods: {}, hooks: {} };

        // Iterate over constructor properties
        for (var prop in this.constructor) {
          // Convert all functions as "classMethods" or "hooks"
          if (typeof this.constructor[prop] === 'function') {
            if (blacklist.hookProps.indexOf(prop) === -1) {
              options.classMethods[prop] = this.constructor[prop];
            } else {
              options.hooks[prop] = this.constructor[prop];
            }
          }
          // Convert the rest as "options"
          else {
            options[prop] = this.constructor[prop];
          }
        }

        // Iterate over instance properties
        for (var prop in this) {
          // Convert all functions as "instanceMethods"
          if (typeof this[prop] === 'function') {
            options.instanceMethods[prop] = this[prop];
          }
          // Convert the rest as "attributes"
          else {
            attributes[prop] = this[prop];
          }
        }

        // Return list of arguments for "sequelize.define" function
        return [modelName, attributes, options];
      }
    });

    return Model;
  };
}

module.exports = exports['default'];